'use strict';

exports.__esModule = true;
exports.default = checkArgs;

var _ordinal = require('./ordinal.js');

var _ordinal2 = _interopRequireDefault(_ordinal);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Validation helper
function checkArgs(name, args) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$nullable = _ref.nullable,
      nullable = _ref$nullable === undefined ? false : _ref$nullable,
      _ref$minArgs = _ref.minArgs,
      minArgs = _ref$minArgs === undefined ? 1 : _ref$minArgs,
      _ref$maxArgs = _ref.maxArgs,
      maxArgs = _ref$maxArgs === undefined ? 1 : _ref$maxArgs;

  if (minArgs === maxArgs && args.length !== minArgs) {
    var plural = minArgs === 1 ? '' : 's';
    throw new Error(name + ' must receive exactly ' + minArgs + ' argument' + plural);
  }
  if (args.length < minArgs) {
    var plural1 = minArgs === 1 ? '' : 's';
    throw new Error(name + ' must receive at least ' + minArgs + ' argument' + plural1 + '.');
  }
  if (args.length > maxArgs) {
    var plural2 = maxArgs === 1 ? '' : 's';
    throw new Error(name + ' accepts at most ' + maxArgs + ' argument' + plural2 + '.');
  }
  for (var i = 0; i < args.length; i++) {
    if (!nullable && args[i] === null) {
      var ordinality = maxArgs !== 1 ? ' ' + (0, _ordinal2.default)(i + 1) : '';
      throw new Error('The' + ordinality + ' argument to ' + name + ' must be non-null');
    }
    if (args[i] === undefined) {
      throw new Error('The ' + (0, _ordinal2.default)(i + 1) + ' argument to ' + name + ' must be defined');
    }
  }
}
//# sourceMappingURL=check-args.js.map