import { Component } from '@angular/core';
import { NavController, NavParams } from 'ionic-angular';

import { VarsGlobales } from '../../providers/vars-globales';
import { SigaService } from '../../providers/siga-service';
import { AlertController } from 'ionic-angular';

/*
  Generated class for the DesempenoCalifica page.

  See http://ionicframework.com/docs/v2/components/#navigation for more info on
  Ionic pages and navigation.
*/
@Component({
  selector: 'page-desempeno-califica',
  templateUrl: 'desempeno-califica.html',
  providers: [SigaService]
})
export class DesempenoCalifica {
  //Variables locales de la clase
  docAprendiz;
  docUsuario;
  noFicha;
  bddCalifica;
  bddAccion;
  aprendiz;
  nomPrograma;
  isenabled:boolean=false;
  input;
  tituloAcc;
  myDateI: String = new Date().toISOString().substring(0, 10);
  myDateF = this.myDateI;
  incremento;
  index;
  
  
  
  

  constructor(public navCtrl: NavController, public varsGlobales: VarsGlobales, 
  							public servicios: SigaService, public alertCtrl:AlertController,
                private navParams: NavParams) {
      //Almacena la información en las variables locales, de las variables globales
      this.docAprendiz = varsGlobales.getDocAprendiz();
      this.docUsuario = varsGlobales.getDocUsuario();
      console.log("Doc Aprendiz: "+this.docAprendiz);
      this.noFicha = varsGlobales.getFicha();
      this.nomPrograma = varsGlobales.getNomPrograma();
      this.aprendiz = navParams.get('aprendiz');
      console.log("Aprendiz: "+this.aprendiz);

      //Busca en la base de datos, las calificaciones que tiene el aprendiz por Instructor y Global
      this.servicios. getBddCalifica(this.docAprendiz, this.noFicha, this.docUsuario)
        .then(
          data => {
            
              console.log(data);
              this.bddCalifica = data;
              //this.aprendiz = data[0].apNombre + " " + data[0].apApellido;
              //console.log("Aprendiz: "+this.aprendiz)
            
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )

         //Busca en la base de datos, las calificaciones de las acciones de mejora
        this.servicios. getBddMuestraAccion(this.docAprendiz, this.noFicha, this.docUsuario)
        .then(
          data => {
            console.log(data);
            this.bddAccion = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
  }

  

  ionViewDidLoad() {
    console.log('ionViewDidLoad DesempenoCalifica');

  }
  

  //Crea un arreglo con la información a guardar en la base de datos
  getDatos(index){
    var datos = {};
    datos["deIdAprendiz"] = this.docAprendiz;
    datos["deIdFicha"] = this.noFicha;
    datos["deIdActividad"] = this.bddCalifica[index].deIdActividad;
    return datos;
  }

  //Actualiza la calificación en la base de datos de A -> Aprobado
  aprobo(index){
    console.log("Aprobo: "+index);
    this.servicios.setBddCalificaAp(this.getDatos(index));
    this.navCtrl.remove(1);
  }

  //Actualiza la calificación en la base de datos de D-> Desaprobado

  noAprobo(index){
    console.log("No Aprobo: "+index);
    this.servicios.setBddCalificaNAp(this.getDatos(index));
    this.isenabled=true;
    this.index = index;
  }


  
  //Toma la información de auto incremento para almacenarla en la base de datos de Desempeño
autoIncremento(data){
    
    var datos2 = {};
    this.incremento = parseInt(data)-1;
    datos2["deIdAccion"] = this.incremento;
    datos2["deIdAprendiz"] = this.docAprendiz;
    datos2["deIdFicha"] = this.noFicha;
    datos2["deIdActividad"] = this.bddCalifica[this.index].deIdActividad;
    this.servicios.setBddDesempenoAM(datos2)
        .then(
          data => {
            console.log(data);
            if(data["response"] == "ok"){
                let alert = this.alertCtrl.create(
                {
                  title:"Aviso",
                  subTitle: "Registro añadido",
                  buttons: ["ok"]
                }
              );
              alert.present();
            }
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
    }

    

        
    
  
/*
  Almacena en un arreglo la información que se almacena en la base de datos de Acción de Mejora
  Con la información de el título y las fechas de entrega
*/
  guardar(){
    var datos = {};
    datos["amTituloMejora"] = this.tituloAcc;
    datos["amFechaInicio"] = this.myDateI;
    datos["amFechaFin"] = this.myDateF;
    console.log(datos);
    this.servicios.setBddAccionMejora(datos)
        .then(
          data => {
            console.log("Este es data: "+data);
            //this.bddRespuesta = data;
            this.autoIncremento(data[0].AUTO_INCREMENT)            
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
        this.navCtrl.remove(1);
}

/*
  Califica la Acción de mejora, agregando la información en un arreglo 
  para luego almacenarla en la bases de datos
*/


calificaAccion(index, arg){
  console.log("Accion: "+this.bddAccion[index].amId);
  console.log("Index: "+index);
  
    var datos = {};
    datos["deApruebaM"] = arg;
    datos["deIdAprendiz"] = this.docAprendiz;
    datos["deIdAccion"] = this.bddAccion[index].amId;
    
    console.log(datos);
    this.servicios.setBddCalificaAcc(datos)
        .then(
          data => {
            console.log(data);
            if(data["response"] == "ok"){
                let alert = this.alertCtrl.create(
                {
                  title:"Aviso",
                  subTitle: "Registro añadido",
                  buttons: ["ok"]
                }
              );
              alert.present();
            }
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
     this.navCtrl.remove(1);   
}

aproboAccion(index){
  this.calificaAccion(index,"A");
}

noAproboAccion(index){
  this.calificaAccion(index,"D");
}
  

}
