import { Component } from '@angular/core';
import { NavController } from 'ionic-angular';

import { VarsGlobales } from '../../providers/vars-globales';
import { SigaService } from '../../providers/siga-service';
import { AlertController } from 'ionic-angular';

import { AprendizFoto } from '../aprendiz-foto/aprendiz-foto';

/*
  Generated class for the ListaDetalle page.

  See http://ionicframework.com/docs/v2/components/#navigation for more info on
  Ionic pages and navigation.
*/

@Component({
  selector: 'page-lista-detalle',
  templateUrl: 'lista-detalle.html',
  providers: [SigaService]
})


export class ListaDetalle {
//Variables de la clase

  aprendiz;
  instructor;
  nomPrograma;
  bddAprendiz;
  bddInasistenciaG;
  bddInasistenciaU;
  bddDesempenoG;
  bddDesempenoGA;
  bddDesempenoGD;
  bddDesempenoU;
  bddDesempenoUA;
  bddDesempenoUD;
  noInasistencia;
  telefono;
  email;
  varDesempenoGA;
  varDesempenoG;
  ficha;
  

  constructor(public navCtrl: NavController, public varsGlobales: VarsGlobales, 
  							public servicios: SigaService, public alertCtrl:AlertController) {
    //Asignación en las variables locales la información de las variables Globales
    this.aprendiz = varsGlobales.getDocAprendiz();
    this.instructor = varsGlobales.getDocUsuario();
    this.ficha = varsGlobales.getFicha();
    this.nomPrograma = varsGlobales.getNomPrograma();

    //Obtiene de la base de datos de aprendiz la información requerida
    this.servicios. getBddAprendiz(this.aprendiz)
        .then(
          data => {
            console.log(data);
            this.bddAprendiz = data;
            this.telefono = data[0].apTelefono;    
            this.email = data[0].apCorreo;    
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )

        

        this.inasistenciaG();
        this.inasistenciaU();
        this.desempenoG();
        this.desempenoU();
        this.desempenoUA();

  }

  ionViewDidLoad() {
    console.log('ionViewDidLoad ListaDetalle');
  }

  //Consulta en la base de datos la inasistencia General
  inasistenciaG(){
    this.servicios. getBddInasistenciaG(this.aprendiz)
        .then(
          data => {
            console.log(data);
            this.bddInasistenciaG = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
  }

  //Consulta en la base de datos la inasistencia por Instructor
  inasistenciaU(){
    this.servicios. getBddInasistenciaU(this.aprendiz, this.instructor)
        .then(
          data => {
            console.log(data);
            this.bddInasistenciaU = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
  }

  //Consulta en la base de datos el desempeño General 
  desempenoG(){
   this.servicios.getBddDesempenoG(this.aprendiz)
        .then(
          data => {
            console.log("DesempeñoG: "+data);
            this.bddDesempenoG = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        ) 

        console.log("DesempeñoG con variabe bdd: "+this.bddDesempenoG);

   //Consulta en la base de datos el desempeño General del aprendiz
   this.servicios.getBddDesempenoGA(this.aprendiz)
        .then(
          data => {
            console.log(data);
            this.bddDesempenoGA = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )

    //Consulta en la base de datos el desempeño General del aprendiz
   this.servicios.getBddDesempenoGD(this.aprendiz)
        .then(
          data => {
            console.log(data);
            this.bddDesempenoGD = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
  }

  //Consulta en la base de datos el desempeño General del aprendiz por el instructor
  desempenoU(){
    this.servicios. getBddDesempenoU(this.aprendiz, this.instructor)
        .then(
          data => {
            console.log(data);
            this.bddDesempenoU = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
  }



  desempenoUA(){
    this.servicios. getBddDesempenoUA(this.aprendiz, this.instructor)
        .then(
          data => {
            console.log(data);
            this.bddDesempenoUA = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )

    this.servicios. getBddDesempenoUD(this.aprendiz, this.instructor)
        .then(
          data => {
            console.log(data);
            this.bddDesempenoUD = data;
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
  }

  editFoto(){
    console.log("Edit Foto");
    console.log("Aprendiz en lista detalle: "+this.aprendiz);
    this.navCtrl.push(AprendizFoto);
  }


  //Actualiza la información del aprendiz del Correo y telefono
  guardar(){
    
    var datos = {};
    datos["apTelefono"] = this.telefono;
    datos["apCorreo"] = this.email;
    datos["apId"] = this.aprendiz;
    this.servicios.setActualizaDatos(datos)
        .then(
          data => {
            console.log(data);
            if(data["response"] == "ok"){
                let alert = this.alertCtrl.create(
                {
                  title:"Aviso",
                  subTitle: "Registro Actualizado",
                  buttons: ["ok"]
                }
              );
              alert.present();
            }
          }
        )
        .catch(
          error => {
            console.log(error);
          }
        )
        this.navCtrl.remove(1);
  }
}
